-- CONSTANTS *************************************************************************************

BarometricAltitude = "BARO"

redMinInterceptSpeed = 950
redMaxInterceptSpeed = 1200

blueCoalition = "blue"

RedBorder = ZONE_POLYGON:NewFromGroupName( "Red Border")
BlueBorder = ZONE_POLYGON:NewFromGroupName( "Blue Border" )

-- RED SIDE **************************************************************************************

RedDetectionSetGroup = SET_GROUP:New()
RedDetectionSetGroup:FilterPrefixes( { "Red EWR Beslan", "Red EWR Gudauta", "Red EWR AWACS" } )
RedDetectionSetGroup:FilterStart()

RedDetectionArea = DETECTION_AREAS:New( RedDetectionSetGroup, 30000 )

RedDispatcher = AI_A2A_DISPATCHER:New( RedDetectionArea )
RedDispatcher:SetTacticalDisplay( false ) -- set to true for debugging
RedDispatcher:SetEngageRadius( 80000 )
RedDispatcher:SetDisengageRadius( 100000 )
RedDispatcher:SetBorderZone( RedBorder )

-- RUS Mozdok SQUADRON & CAP *******************************************

MozdokSqdn = "RUS Mozdok"
RedDispatcher:SetSquadron( MozdokSqdn, AIRBASE.Caucasus.Mozdok, { "RUS CAP Mozdok Mig-23", "RUS CAP Mozdok Mig-25" }, 32 )
RedDispatcher:SetSquadronOverhead( MozdokSqdn, 1 )
RedDispatcher:SetSquadronGrouping( MozdokSqdn, 2 )
RedDispatcher:SetSquadronFuelThreshold( MozdokSqdn, 0.30 )
RedDispatcher:SetSquadronLandingNearAirbase( MozdokSqdn )

RedMozdokCAPRoute = ZONE_POLYGON:NewFromGroupName( "RUS Mozdok CAP Route")
RedDispatcher:SetSquadronCap( MozdokSqdn, RedMozdokCAPRoute, 4000, 8000, 850, 950, redMinInterceptSpeed, redMaxInterceptSpeed, BarometricAltitude )
RedDispatcher:SetSquadronCapInterval( MozdokSqdn, 3, 300, 600, 1 )
RedDispatcher:SetSquadronGci( MozdokSqdn, redMinInterceptSpeed, redMaxInterceptSpeed )

-- ABK Gudauta SQUADRON & CAP *******************************************

GudautaSqdn = "ABK Gudauta"
RedDispatcher:SetSquadron( GudautaSqdn, AIRBASE.Caucasus.Gudauta, { "ABK CAP Gudauta Mig-21" }, 16 )
RedDispatcher:SetSquadronOverhead( GudautaSqdn, 1 )
RedDispatcher:SetSquadronGrouping( GudautaSqdn, 2 )
RedDispatcher:SetSquadronFuelThreshold( GudautaSqdn, 0.30 )
RedDispatcher:SetSquadronLandingNearAirbase( GudautaSqdn )

RedGudautaCAPRoute = ZONE_POLYGON:NewFromGroupName( "ABK Gudauta CAP Route")
RedDispatcher:SetSquadronCap( GudautaSqdn, RedGudautaCAPRoute, 1500, 6000, 850, 950, redMinInterceptSpeed, redMaxInterceptSpeed, BarometricAltitude )
RedDispatcher:SetSquadronCapInterval( GudautaSqdn, 1, 300, 600, 1 )
RedDispatcher:SetSquadronGci( GudautaSqdn, redMinInterceptSpeed, redMaxInterceptSpeed )

-- RED SIDE CAS AI Mi-24 **************************************************************************************

RedEngagementZoneMi24 = ZONE:New( "Conflict Zone Red Mi-24" )
RedPatrolZoneMi24 = ZONE:New( "Red CAS Mi-24 Loiter" )

RedTargets1 = GROUP:FindByName("GEO Armoured Team #1")
RedTargets2 = GROUP:FindByName("GEO Armoured Team #2")
RedTargets3 = GROUP:FindByName("GEO Armoured Team #3")

RedAICasZoneMi24 = AI_CAS_ZONE:New( RedPatrolZoneMi24, 500, 1000, 500, 600, RedEngagementZoneMi24, "RADIO" )
RedAICasZoneMi24:ManageDamage(0.8)
RedAICasZoneMi24:ManageFuel(0.1, 0)

-- Check every 60 seconds whether the Targets have been eliminated.
-- When the trigger completed has been fired, the Plane will go back to the Patrol Zone.
Check, CheckScheduleID = SCHEDULER:New(nil,
  function()
    if RedTargets1:IsAlive() or RedTargets2:IsAlive() or RedTargets3:IsAlive() then
    	totalTargets = RedTargets1:GetSize() + RedTargets2:GetSize() + RedTargets3:GetSize()
      BASE:E( "RED CAS Mission Mi-24: " .. totalTargets .. " targets left to be destroyed.")
    else
      BASE:E( "RED CAS Mission Mi-24: The required targets are destroyed." )
      RedAICasZoneMi24:__Accomplish( 1 ) -- Now they should fly back to the patrolzone and patrol.
    end
  end, {}, 20, 60, 0.2 )

-- When the targets in the zone are destroyed, (see scheduled function), the planes will return home ...
function RedAICasZoneMi24:OnAfterAccomplish( Controllable, From, Event, To )
  BASE:E( "RED CAS Mission Mi-24: Sending the Mi-24 back to base." )
  Check:Stop( CheckScheduleID )
  RedAICasZoneMi24:__RTB( 1 )
end

CASAbkSpawnMi24 = SPAWN
  :New( "ABK Mi-24" )
  :InitLimit(2,0)
  :InitRepeatOnEngineShutDown()
  :SpawnScheduled(60,0)
  :OnSpawnGroup(
    --- @param Wrapper.Group#GROUP SpawnGroup
    function( SpawnGroup )
      RedAICasZoneMi24:SetControllable( SpawnGroup )
      RedAICasZoneMi24:__Start( 1 )
      RedAICasZoneMi24:__Engage( 120 )
    end
  )

-- RED SIDE CAS AI Su-25 **************************************************************************************

RedEngagementZoneSu25 = ZONE:New( "Conflict Zone Red Su-25" )
RedPatrolZoneSu25 = ZONE:New( "Red CAS Su-25 Loiter" )

RedTargets3 = GROUP:FindByName("GEO Armoured Team #3")
RedTargets4 = GROUP:FindByName("GEO Armoured Team #4")

RedAICasZoneSu25 = AI_CAS_ZONE:New( RedPatrolZoneSu25, 500, 1000, 500, 600, RedEngagementZoneSu25, "RADIO" )
RedAICasZoneSu25:ManageDamage(0.8)
RedAICasZoneSu25:ManageFuel(0.3, 0)

-- Check every 60 seconds whether the Targets have been eliminated.
-- When the trigger completed has been fired, the Plane will go back to the Patrol Zone.
Check, CheckScheduleID = SCHEDULER:New(nil,
  function()
    if RedTargets3:IsAlive() or RedTargets4:IsAlive() then
    	totalTargets = RedTargets3:GetSize() + RedTargets4:GetSize()
      BASE:E( "RED CAS Mission Su-25: " .. totalTargets .. " targets left to be destroyed.")
    else
      BASE:E( "RED CAS Mission Su-25: The required targets are destroyed." )
      RedAICasZoneSu25:__Accomplish( 1 ) -- Now they should fly back to the patrolzone and patrol.
    end
  end, {}, 20, 60, 0.2 )

-- When the targets in the zone are destroyed, (see scheduled function), the planes will return home ...
function RedAICasZoneSu25:OnAfterAccomplish( Controllable, From, Event, To )
  BASE:E( "RED CAS Mission Su-25: Sending the Su-25 back to base." )
  Check:Stop( CheckScheduleID )
  RedAICasZoneSu25:__RTB( 1 )
end

CASAbkSpawnSu25 = SPAWN
  :New( "ABK Su-25" )
  :InitLimit(2,0)
  :InitRepeatOnEngineShutDown()
  :SpawnScheduled(60,0)
  :OnSpawnGroup(
    --- @param Wrapper.Group#GROUP SpawnGroup
    function( SpawnGroup )
      RedAICasZoneSu25:SetControllable( SpawnGroup )
      RedAICasZoneSu25:__Start( 1 )
      RedAICasZoneSu25:__Engage( 120 )
    end
  )

-- BLUE SIDE CAP AI **************************************************************************************

BlueDetectionSetGroup = SET_GROUP:New()
BlueDetectionSetGroup:FilterPrefixes( { "Blue EWR Batumi", "Blue EWR Vaziani" } )
BlueDetectionSetGroup:FilterStart()

BlueDetectionArea = DETECTION_AREAS:New( BlueDetectionSetGroup, 30000 )

BlueDispatcher = AI_A2A_DISPATCHER:New( BlueDetectionArea )
BlueDispatcher:SetTacticalDisplay( false ) -- set to true for debugging
BlueDispatcher:SetEngageRadius( 80000 )
BlueDispatcher:SetDisengageRadius( 100000 )
BlueDispatcher:SetBorderZone( BlueBorder)

BlueBatumiCAPRoute = ZONE_POLYGON:NewFromGroupName( "Blue Batumi CAP Route")

-- TUR SQUADRON & CAP *******************************************
TurkishSqdn = "TUR Batumi"
BlueDispatcher:SetSquadron( TurkishSqdn, AIRBASE.Caucasus.Batumi, { "TUR CAP F-4" }, 8 )
BlueDispatcher:SetSquadronOverhead( TurkishSqdn, 1 )
BlueDispatcher:SetSquadronGrouping( TurkishSqdn, 2 )
BlueDispatcher:SetSquadronFuelThreshold( TurkishSqdn, 0.30 )
BlueDispatcher:SetSquadronLandingNearAirbase( TurkishSqdn )

BlueDispatcher:SetSquadronCap( TurkishSqdn, BlueBatumiCAPRoute, 4000, 8000, 800, 950, 950, 1200, BarometricAltitude )
BlueDispatcher:SetSquadronCapInterval( TurkishSqdn, 1, 480, 600, 1 )

-- US SQUADRON & CAP *******************************************
UsSqdn = "US Batumi"
BlueDispatcher:SetSquadron( UsSqdn, AIRBASE.Caucasus.Batumi, { "US CAP F-16" }, 8 )
BlueDispatcher:SetSquadronOverhead( UsSqdn, 1 )
BlueDispatcher:SetSquadronGrouping( UsSqdn, 2 )
BlueDispatcher:SetSquadronFuelThreshold( UsSqdn, 0.30 )
BlueDispatcher:SetSquadronLandingNearAirbase( UsSqdn )

BlueDispatcher:SetSquadronCap( UsSqdn, BlueBatumiCAPRoute, 4000, 8000, 850, 950, 1000, 1200, BarometricAltitude )
BlueDispatcher:SetSquadronCapInterval( UsSqdn, 1, 180, 300, 1 )

-- BLUE SIDE CAS AI Su-25 EAST **************************************************************************************

BlueCASEngagementZoneEast = ZONE:New( "Conflict Zone E" )
BluePatrolZoneEast = ZONE:New( "Blue CAS Loiter E" )

BlueTargets1 = GROUP:FindByName("RUS Armoured Team #1")
BlueTargets2 = GROUP:FindByName("RUS Armoured Team #2")

BlueAICasZoneEast = AI_CAS_ZONE:New( BluePatrolZoneEast, 500, 1000, 500, 600, BlueCASEngagementZoneEast, "RADIO" )
BlueAICasZoneEast:ManageDamage(0.8)
BlueAICasZoneEast:ManageFuel(0.3, 0)

-- Check every 60 seconds whether the Targets have been eliminated.
-- When the trigger completed has been fired, the Plane will go back to the Patrol Zone.
Check, CheckScheduleID = SCHEDULER:New(nil,
  function()
    if BlueTargets1:IsAlive() or BlueTargets2:IsAlive() then
    	totalTargets = BlueTargets1:GetSize() + BlueTargets2:GetSize()
      BASE:E( "BLUE CAS Mission East: " .. totalTargets .. " targets left to be destroyed.")
    else
      BASE:E( "BLUE CAS Missionm East: The required targets are destroyed." )
      BlueAICasZoneEast:__Accomplish( 1 ) -- Now they should fly back to the patrolzone and patrol.
    end
  end, {}, 20, 60, 0.2 )

-- When the targets in the zone are destroyed, (see scheduled function), the planes will return home ...
function BlueAICasZoneEast:OnAfterAccomplish( Controllable, From, Event, To )
  BASE:E( "BLUE CAS Mission East: Sending the Su-25 back to base." )
  Check:Stop( CheckScheduleID )
  BlueAICasZoneEast:__RTB( 1 )
end

CASGeoSpawnEast = SPAWN
  :New( "GEO CAS Su-25" )
  :InitLimit(2,0)
  :InitRepeatOnEngineShutDown()
  :SpawnScheduled(60,0)
  :OnSpawnGroup(
    --- @param Wrapper.Group#GROUP SpawnGroup
    function( SpawnGroup )
      BlueAICasZoneEast:SetControllable( SpawnGroup )
      BlueAICasZoneEast:__Start( 1 )
      BlueAICasZoneEast:__Engage( 120 )
    end
  )

-- BLUE SIDE CAS AI Mi-24 WEST **************************************************************************************

BlueCASEngagementZoneWest = ZONE:New( "Conflict Zone W" )
BluePatrolZoneWest = ZONE:New( "Blue CAS Loiter W" )

BlueTargets3 = GROUP:FindByName("RUS Armoured Team #3")
BlueTargets4 = GROUP:FindByName("RUS Armoured Team #4")

BlueAICasZoneWest = AI_CAS_ZONE:New( BluePatrolZoneWest, 500, 1000, 500, 600, BlueCASEngagementZoneWest, "RADIO" )
BlueAICasZoneWest:ManageDamage(0.8)
BlueAICasZoneWest:ManageFuel(0.1, 0)

-- Check every 60 seconds whether the Targets have been eliminated.
-- When the trigger completed has been fired, the Plane will go back to the Patrol Zone.
Check, CheckScheduleID = SCHEDULER:New(nil,
  function()
    if BlueTargets3:IsAlive() or BlueTargets4:IsAlive() then
    	totalTargets = BlueTargets3:GetSize() + BlueTargets4:GetSize()
      BASE:E( "BLUE CAS Mission West: " .. totalTargets .. " targets left to be destroyed.")
    else
      BASE:E( "BLUE CAS Mission West: The required targets are destroyed." )
      BlueAICasZoneWest:__Accomplish( 1 ) -- Now they should fly back to the patrolzone and patrol.
    end
  end, {}, 20, 60, 0.2 )

-- When the targets in the zone are destroyed, (see scheduled function), the planes will return home ...
function BlueAICasZoneWest:OnAfterAccomplish( Controllable, From, Event, To )
  BASE:E( "BLUE CAS Mission West: Sending the Mi-24 back to base." )
  Check:Stop( CheckScheduleID )
  BlueAICasZoneWest:__RTB( 1 )
end

CASGeoSpawnWest = SPAWN
  :New( "GEO CAS Mi-24" )
  :InitLimit(2,0)
  :InitRepeatOnEngineShutDown()
  :SpawnScheduled(60,0)
  :OnSpawnGroup(
    --- @param Wrapper.Group#GROUP SpawnGroup
    function( SpawnGroup )
      BlueAICasZoneWest:SetControllable( SpawnGroup )
      BlueAICasZoneWest:__Start( 1 )
      BlueAICasZoneWest:__Engage( 120 )
    end
  )

-- A2A TASKING ***************************************************************************************

USHQ = GROUP:FindByName( "Blue HQ", "Zeus HQ" )
CommandCenter = COMMANDCENTER:New(USHQ, "Zeus")

Scoring = SCORING:New("Operation Dauntless Resolution")

-- Air to Air Tasking *******************************************
InterceptMission = MISSION
  :New( CommandCenter, "Intercept", "", "Intercept", coalition.side.BLUE )
  :AddScoring( Scoring )

PlayerGroup = SET_GROUP:New():FilterCoalitions( blueCoalition ):FilterPrefixes( "US Player" ):FilterStart()

-- Setup the A2A dispatcher, and initialize it.
InterceptDispatcher = TASK_A2A_DISPATCHER:New( InterceptMission, PlayerGroup, BlueDetectionArea )
InterceptDispatcher:SetRefreshTimeInterval( 30 )

 --- @param #TaskDispatcher self
 -- @param #string From Contains the name of the state from where the Event was triggered.
 -- @param #string Event Contains the name of the event that was triggered. In this case Assign.
 -- @param #string To Contains the name of the state that will be transitioned to.
 -- @param Tasking.Task_A2A#TASK_A2A Task The Task object, which is any derived object from TASK_A2A.
 -- @param Wrapper.Unit#UNIT TaskUnit The Unit or Client that contains the Player.
 -- @param #string PlayerName The name of the Player that joined the TaskUnit.
 function InterceptDispatcher:OnAfterAssign( From, Event, To, Task, TaskUnit, PlayerName )
   Task:SetScoreOnProgress( PlayerName, 20, TaskUnit )
   Task:SetScoreOnSuccess( PlayerName, 100, TaskUnit )
   Task:SetScoreOnFail( PlayerName, -5, TaskUnit )
 end

 -- A2G TASKING ***************************************************************************************

CasBaiMission = MISSION
	:New( CommandCenter, "SEAD/BAI/CAS", "", "SEAD/BAI/CAS", coalition.side.BLUE )
	:AddScoring( Scoring )

 -- Define the RecceSet that will detect the enemy.
USFacSet = SET_GROUP:New():FilterCoalitions( blueCoalition ):FilterPrefixes( "US FAC" ):FilterStart()
-- Setup the detection.
-- UsFacDetectionUnits = DETECTION_UNITS:New( USFacSet )
UsFacDetectionUnits = DETECTION_AREAS:New( USFacSet, 1000 )
UsFacDetectionUnits:FilterCategories({Unit.Category.AIRPLANE,Unit.Category.HELICOPTER})
UsFacDetectionUnits:__Start( 5 )

-- Setup Designation for the AttackSet.
UsFacDetectionUnitsDesignator = DESIGNATE:New( CommandCenter, UsFacDetectionUnits, PlayerGroup )
-- Generate the random laser codes.
UsFacDetectionUnitsDesignator:GenerateLaserCodes()
-- The Player has a predefined 1666 laser code in the mission editor
UsFacDetectionUnitsDesignator:AddMenuLaserCode( 1666, "*** USE 1666 ***" )
UsFacDetectionUnitsDesignator:SetThreatLevelPrioritization( true )

CasBaiDispatcher = TASK_A2G_DISPATCHER:New( CasBaiMission, PlayerGroup, UsFacDetectionUnits )
CasBaiDispatcher:SetRefreshTimeInterval( 30 )

 --- @param #TaskInterceptDispatcher self
 -- @param #string From Contains the name of the state from where the Event was triggered.
 -- @param #string Event Contains the name of the event that was triggered. In this case Assign.
 -- @param #string To Contains the name of the state that will be transitioned to.
 -- @param Tasking.Task_A2A#TASK_A2A Task The Task object, which is any derived object from TASK_A2A.
 -- @param Wrapper.Unit#UNIT TaskUnit The Unit or Client that contains the Player.
 -- @param #string PlayerName The name of the Player that joined the TaskUnit.
 function CasBaiDispatcher:OnAfterAssign( From, Event, To, Task, TaskUnit, PlayerName )
   Task:SetScoreOnProgress( PlayerName, 20, TaskUnit )
   Task:SetScoreOnSuccess( PlayerName, 100, TaskUnit )
   Task:SetScoreOnFail( PlayerName, -5, TaskUnit )
 end

collectgarbage()
BASE:E( "Collected garbage" )